function [W, funcVal] = Least_SRMTL1(X, Y, R, rho1, rho2, opts)
% sparse structure-regularized learning with least squares loss.
% By Jiayu Zhou (jiayu.zhou@asu.edu) 

% Objective:
% argmin_W { sum_i^t (0.5 * norm (Y{i} - X{i}' * W(:, i))^2) 
%            + rho1 * norm(W*R, 'fro')^2 + rho2 * \|W\|_1}

% R encodes structure relationship which is similarity
% 1)Structure order is given by using [1 -1 0 ...; 0 1 -1 ...; ...]
%    e.g.: R=zeros(t,t-1);H(1:(t+1):end)=1;H(2:(t+1):end)=-1;
% 2)Ridge penalty term by setting: R = eye(t)
% 3)All related regularized: R = eye (t) - ones (t) / t

%%% input 
% X: {d * n} * t - input matrix
% Y: {n * 1} * t - output matrix
% R: regularization structure
% rho1: structure regularization parameter
% rho2: sprasity controlling parameter

%%% output
% W: model: d * t
% funcVal: function value vector.

if nargin <6
    opts = [];
end

if isfield(opts, 'max_iter')
    max_iter = opts.max_iter;
else
    max_iter = 100;
end

if isfield(opts, 'tol_funcVal')
    tol_funcVal = opts.tol_funcVal;
else
    tol_funcVal = 10^-5;
end


funcVal = [];

if rho1==0
   task_num  = 1;
   dimension = size(X, 1);
else
   task_num  = length (X);
   dimension = size(X{1}, 1);
end

% precomputation.
RRt = R * R';


% initial W
W0 = zeros(dimension, task_num);

%Ws= W0;
Wz= W0;
Wz_old = W0;


t = 1;
t_old = 0;


iter = 0;
gamma = 1;
gamma_inc = 2; 



while iter < max_iter
    alpha = (t_old - 1) /t;
    
    Ws = (1 + alpha) * Wz - alpha * Wz_old;

    % compute function value and gradients of the search point
    gWs  = gradVal_eval(Ws, rho1);
    Fs   = funVal_eval  (Ws, rho1);
    
    while true
        [Wzp l1c_wzp] = l1_projection(Ws - gWs/gamma, 2 * rho2 / gamma);
        Fzp = funVal_eval  (Wzp, rho1);
        
        delta_Wzp = Wzp - Ws;
        
        Fzp_gamma = Fs + trace(delta_Wzp' * gWs) + gamma/2 * norm(delta_Wzp, 'fro')^2;
        
        if (Fzp <= Fzp_gamma)
            break;
        else
            gamma = gamma * gamma_inc;
        end
    end
    
    Wz_old = Wz;
    Wz = Wzp;
    
    funcVal = cat(1, funcVal, Fzp + rho2 * l1c_wzp);
    
    
    % test stop condition.
    if length(funcVal)> 1 &&...
            (abs(funcVal(end-1) - funcVal(end))/ funcVal(end))<=tol_funcVal
        break;
    end
    
    iter = iter + 1;
    t_old = t;
    t = 0.5 * (1 + (1+ 4 * t^2)^0.5);
    
end

W = Wzp;


% private functions

function [z l1_comp_val] = l1_projection (v, beta)
        % this projection calculates
        % argmin_z = \|z-v\|_2^2 + beta \|z\|_1
        % z: solution
        % l1_comp_val: value of l1 component (\|z\|_1)
        
        z = zeros(size(v));
        vp = v - beta/2;
        z (v> beta/2)  = vp(v> beta/2);
        vn = v + beta/2;
        z (v< -beta/2) = vn(v< -beta/2);
    
        
        l1_comp_val = sum(sum(abs(z)));
    end


    function [grad_W] = gradVal_eval(W, rho1)
        grad_W = [];
        if task_num==1
            grad_W = cat(2, grad_W, X*(X' * W-Y) );
        else
           for i = 1:task_num
            grad_W = cat(2, grad_W, X{i}*(X{i}' * W(:,i)-Y{i}) );
           end
           grad_W = grad_W+ rho1 * 2 *  W * RRt;
        end
        
    end

    function [funcVal] = funVal_eval (W, rho1)
        funcVal = 0;
        if task_num==1  % for single task
            funcVal = funcVal + 0.5 * norm (Y - X' * W)^2;
        else
          for i = 1: task_num
            funcVal = funcVal + 0.5 * norm (Y{i} - X{i}' * W(:, i))^2;
          end
          funcVal = funcVal + rho1 * norm(W*R, 'fro')^2;
        end
        
    end



end